<?php

namespace Drupal\translation_alias\Commands;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\path_alias\AliasRepositoryInterface;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Output\ConsoleOutput;

/**
 * Drush commands for managing path aliases translations.
 */
class TranslationAliasCommands extends DrushCommands {

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The path alias repository.
   *
   * @var \Drupal\path_alias\AliasRepositoryInterface
   */
  protected $aliasRepository;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new TranslationAliasCommands object.
   */
  public function __construct(
    AliasManagerInterface $alias_manager,
    AliasRepositoryInterface $alias_repository,
    Connection $database,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct();
    $this->aliasManager = $alias_manager;
    $this->aliasRepository = $alias_repository;
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Gets all existing aliases for a path and language.
   *
   * @param string $path
   *   The system path.
   * @param string $langcode
   *   The language code.
   *
   * @return array
   *   Array of existing aliases with their IDs.
   */
  protected function getExistingAliases($path, $langcode) {
    return $this->database->select('path_alias', 'pa')
      ->fields('pa', ['id', 'alias', 'status'])
      ->condition('path', $path)
      ->condition('langcode', $langcode)
      ->execute()
      ->fetchAllAssoc('id', \PDO::FETCH_ASSOC);
  }

  /**
   * Updates path alias for a single entity translation.
   *
   * @command translation-alias:update-path
   * @aliases ta:up,translation-alias-update
   * @usage translation-alias:update-path 123 node fr /fr/new-path
   */
  public function updatePath($entity_id, $entity_type, $langcode, $new_path_alias) {
    try {
      // Validate entity exists
      $entity_storage = $this->entityTypeManager->getStorage($entity_type);
      $entity = $entity_storage->load($entity_id);
      if (!$entity) {
        throw new \Exception(dt('@type ID @id does not exist.', [
          '@type' => $entity_type,
          '@id' => $entity_id,
        ]));
      }

      // Ensure path starts with a slash
      $new_path_alias = '/' . ltrim($new_path_alias, '/');
      
      // Build the system path
      $path = '/' . $entity_type . '/' . $entity_id;

      // Get all existing aliases for this path/language
      $existing_aliases = $this->getExistingAliases($path, $langcode);

      // Get the path alias storage
      $path_alias_storage = $this->entityTypeManager->getStorage('path_alias');

      if (empty($existing_aliases)) {
        // No aliases exist - create new one
        $path_alias = $path_alias_storage->create([
          'path' => $path,
          'alias' => $new_path_alias,
          'langcode' => $langcode,
        ]);
        $path_alias->save();
        $this->logger()->success(dt('Created new alias @alias', [
          '@alias' => $new_path_alias,
        ]));
      }
      else {
        // Handle existing aliases
        $alias_already_exists = FALSE;

        foreach ($existing_aliases as $alias_data) {
          if ($alias_data['alias'] === $new_path_alias) {
            $alias_already_exists = TRUE;
            if ($alias_data['status'] == 0) {
              // Reactivate this alias
              $path_alias = $path_alias_storage->load($alias_data['id']);
              if ($path_alias) {
                $path_alias->set('status', 1);
                $path_alias->save();
                $this->logger()->success(dt('Reactivated existing alias @alias', [
                  '@alias' => $new_path_alias,
                ]));
              }
            }
            else {
              $this->logger()->notice(dt('Alias @alias already exists and is active', [
                '@alias' => $new_path_alias,
              ]));
            }
          }
          elseif ($alias_data['status'] == 1) {
            // Deactivate other active aliases
            $path_alias = $path_alias_storage->load($alias_data['id']);
            if ($path_alias) {
              $path_alias->set('status', 0);
              $path_alias->save();
              $this->logger()->notice(dt('Deactivated old alias @alias', [
                '@alias' => $alias_data['alias'],
              ]));
            }
          }
        }

        if (!$alias_already_exists) {
          // Create new alias and make it active
          $path_alias = $path_alias_storage->create([
            'path' => $path,
            'alias' => $new_path_alias,
            'langcode' => $langcode,
          ]);
          $path_alias->save();
          $this->logger()->success(dt('Created new alias @alias and made it active', [
            '@alias' => $new_path_alias,
          ]));
        }
      }

      // Clear the path alias cache for this path
      $this->aliasManager->cacheClear($path);
    }
    catch (\Exception $e) {
      $this->logger()->error($e->getMessage());
      throw $e;
    }
  }

  /**
   * Bulk updates path aliases from a CSV file.
   *
   * @command translation-alias:bulk-update
   * @aliases ta:bu,translation-alias-bulk
   * @usage translation-alias:bulk-update path/to/file.csv
   */
  public function bulkUpdate($file) {
    if (!file_exists($file)) {
      throw new \Exception(dt('File @file does not exist.', [
        '@file' => $file,
      ]));
    }

    $handle = fopen($file, 'r');
    if (!$handle) {
      throw new \Exception(dt('Could not open file @file', [
        '@file' => $file,
      ]));
    }

    // Skip header row and validate
    $headers = fgetcsv($handle);
    $required_columns = ['entity_id', 'entity_type', 'langcode', 'new_path_alias'];
    
    if ($headers !== $required_columns) {
      throw new \Exception(dt('Invalid CSV headers. Expected: @columns', [
        '@columns' => implode(', ', $required_columns),
      ]));
    }

    $output = new ConsoleOutput();
    $progress = new ProgressBar($output);
    $progress->start();

    $success_count = 0;
    $error_count = 0;
    $line = 2; // Start counting after header

    while (($data = fgetcsv($handle)) !== FALSE) {
      if (count($data) !== 4) {
        $this->logger()->warning(dt('Invalid number of columns in row @row', [
          '@row' => $line,
        ]));
        $error_count++;
        continue;
      }

      try {
        $this->updatePath(
          (int) $data[0],  // entity_id
          $data[1],        // entity_type
          $data[2],        // langcode
          $data[3]         // new_path_alias
        );
        $success_count++;
        $progress->advance();
      }
      catch (\Exception $e) {
        $this->logger()->warning(dt('Error on line @line: @message', [
          '@line' => $line,
          '@message' => $e->getMessage(),
        ]));
        $error_count++;
      }

      $line++;
    }

    $progress->finish();
    $output->writeln('');
    fclose($handle);

    $this->logger()->success(dt('Bulk update completed with @success successes and @errors errors.', [
      '@success' => $success_count,
      '@errors' => $error_count,
    ]));
  }

}
